@@ -16,7 +16,7 @@ def _get_storage_client(http):
     """Gets a python storage client"""
     os.environ["STORAGE_EMULATOR_HOST"] = "http://localhost:9023"
 
-    # Cloud storage uses environment variables to configure api endpoints for
+    # Cloud Storage uses environment variables to configure API endpoints for
     # file upload - which is read at module import time
     from google.cloud import storage
 
@@ -44,7 +44,7 @@ class ServerBaseCase(BaseTestCase):
     def setUp(self):
         self._session = requests.Session()
         self._client = _get_storage_client(self._session)
-        ObjectsTests._server.wipe()
+        self._server.wipe()
 
 
 class BucketsTests(BaseTestCase):
@@ -58,7 +58,7 @@ class BucketsTests(BaseTestCase):
         cls._server.stop()
 
     def setUp(self):
-        BucketsTests._server.wipe()
+        self._server.wipe()
         self._session = requests.Session()
         self._client = _get_storage_client(self._session)
 
@@ -105,8 +105,7 @@ class BucketsTests(BaseTestCase):
             self.assertFalse(pwd.exists("bucket_name"))
 
     def test_bucket_delete_non_existing(self):
-        # client.bucket doesn't create the actual bucket resource remotely,
-        # it only instantiate it in the local client
+        # client.bucket doesn't create the actual bucket resource remotely
         bucket = self._client.bucket("bucket_name")
         with self.assertRaises(NotFound):
             bucket.delete()
@@ -135,8 +134,6 @@ class BucketsTests(BaseTestCase):
         with fs.open_fs(STORAGE_BASE + STORAGE_DIR) as pwd:
             self.assertFalse(pwd.exists("bucket_name"))
 
-    # TODO: test delete-force
-
 
 class DefaultBucketTests(BaseTestCase):
     def tearDown(self):
@@ -257,6 +254,12 @@ class ObjectsTests(ServerBaseCase):
 
         self.assertIsNone(res)
 
+    def test_download_nonexistent(self):
+        bucket = self._client.create_bucket("testbucket")
+        blob = bucket.blob("idonotexist")
+        with self.assertRaises(NotFound):
+            blob.download_as_bytes()
+
     def test_download_as_bytes(self):
         content = "The quick brown fox jumps over the lazy dog\n"
         bucket = self._client.create_bucket("testbucket")
@@ -268,6 +271,39 @@ class ObjectsTests(ServerBaseCase):
         fetched_content = blob.download_as_bytes()
         self.assertEqual(fetched_content, content.encode("utf-8"))
 
+    def test_download_range_start(self):
+        content = b"123456789"
+        bucket = self._client.create_bucket("testbucket")
+
+        blob = bucket.blob("iexist")
+        blob.upload_from_string(content)
+
+        blob = bucket.get_blob("iexist")
+        fetched_content = blob.download_as_bytes(start=2)
+        self.assertEqual(fetched_content, b"3456789")
+
+    def test_download_range_end(self):
+        content = b"123456789"
+        bucket = self._client.create_bucket("testbucket")
+
+        blob = bucket.blob("iexist")
+        blob.upload_from_string(content)
+
+        blob = bucket.get_blob("iexist")
+        fetched_content = blob.download_as_bytes(end=4)
+        self.assertEqual(fetched_content, b"12345")
+
+    def test_download_range_start_end(self):
+        content = b"123456789"
+        bucket = self._client.create_bucket("testbucket")
+
+        blob = bucket.blob("iexist")
+        blob.upload_from_string(content)
+
+        blob = bucket.get_blob("iexist")
+        fetched_content = blob.download_as_bytes(start=2, end=4)
+        self.assertEqual(fetched_content, b"345")
+
     def test_set_content_encoding(self):
         content = "The quick brown fox jumps over the lazy dog\n"
         bucket = self._client.create_bucket("testbucket")
@@ -357,7 +393,7 @@ class ObjectsTests(ServerBaseCase):
         blob.md5_hash = md5_hash
         blob.upload_from_string(content)
         download_blob = bucket.get_blob("hashtest")
-        self.assertEqual(download_blob.download_as_bytes(), content)
+        self.assertEqual(download_blob.download_as_bytes(checksum="md5"), content)
         self.assertEqual(download_blob.md5_hash, md5_hash)
 
     def test_invalid_md5_hash(self):
@@ -376,7 +412,7 @@ class ObjectsTests(ServerBaseCase):
         blob.crc32c = crc32c_hash
         blob.upload_from_string(content)
         download_blob = bucket.get_blob("hashtest")
-        self.assertEqual(download_blob.download_as_bytes(), content)
+        self.assertEqual(download_blob.download_as_bytes(checksum="crc32c"), content)
         self.assertEqual(download_blob.crc32c, crc32c_hash)
 
     def test_invalid_crc32c_hash(self):
@@ -706,6 +742,30 @@ class HttpEndpointsTest(ServerBaseCase):
         self.assertEqual(response.status_code, 200)
         self.assertEqual(response.content, content.encode("utf-8"))
 
+    def test_download_by_dl_api_url(self):
+        """ Objects should be downloadable over HTTP from the emulator client. """
+        content = "Here is some content 123"
+        bucket = self._client.create_bucket("bucket")
+        blob = bucket.blob("something.txt")
+        blob.upload_from_string(content)
+
+        url = self._url("/download/storage/v1/b/bucket/o/something.txt")
+        response = requests.get(url)
+        self.assertEqual(response.status_code, 200)
+        self.assertEqual(response.content, content.encode("utf-8"))
+
+    def test_download_by_api_media_url(self):
+        """ Objects should be downloadable over HTTP from the emulator client. """
+        content = "Here is some content 456"
+        bucket = self._client.create_bucket("bucket")
+        blob = bucket.blob("something.txt")
+        blob.upload_from_string(content)
+
+        url = self._url("/storage/v1/b/bucket/o/something.txt")
+        response = requests.get(url, params={"alt": "media"})
+        self.assertEqual(response.status_code, 200)
+        self.assertEqual(response.content, content.encode("utf-8"))
+
     def test_download_file_within_folder(self):
         """Cloud Storage allows folders within buckets, so the download URL should allow for this."""
         content = "Here is some content"
